//  -*- Mode: Java -*-
//
// HookList.java

/*
+---------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2014      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK -----------------------------+
*/

package edu.isi.stella;

import edu.isi.stella.javalib.*;

/** Each HOOK-LIST object contains a list of zero or
 * more function codes that get funcall'd some procedure.
 * @author Stella Java Translator
 */
public class HookList extends KeyValueList {
    public String signature;
    public String documentation;

  public static HookList newHookList() {
    { HookList self = null;

      self = new HookList();
      self.theKvList = null;
      self.documentation = null;
      self.signature = null;
      return (self);
    }
  }

  /** Run all hook functions in <code>hooklist</code>, applying
   * each one to <code>argument</code>.
   * @param hooklist
   * @param argument
   */
  public static void runHooks(HookList hooklist, Stella_Object argument) {
    { KvListIterator it = ((KvListIterator)(hooklist.allocateIterator()));

      while (it.nextP()) {
        edu.isi.stella.javalib.Native.funcall(((FunctionCodeWrapper)(((FunctionCodeWrapper)(it.value)))).wrapperValue, null, new java.lang.Object [] {argument});
      }
    }
  }

  /** Remove the function named <code>hookfunction</code> from <code>hooklist</code>.
   * @param hooklist
   * @param hookfunction
   */
  public static void dropHook(HookList hooklist, Symbol hookfunction) {
    hooklist.removeAt(hookfunction);
  }

  /** Insert the function named <code>hookfunction</code> into <code>hooklist</code>.
   * @param hooklist
   * @param hookfunction
   */
  public static void addHook(HookList hooklist, Symbol hookfunction) {
    { MethodSlot function = Symbol.lookupFunction(hookfunction);

      if (function == null) {
        Stella.STANDARD_WARNING.nativeStream.println("Warning: Reference to non-existent hook function: `hookFunction'.");
        return;
      }
      hooklist.insertAt(hookfunction, FunctionCodeWrapper.wrapFunctionCode(function.functionCode));
    }
  }

  public static Stella_Object accessHookListSlotValue(HookList self, Symbol slotname, Stella_Object value, boolean setvalueP) {
    if (slotname == Stella.SYM_STELLA_SIGNATURE) {
      if (setvalueP) {
        self.signature = ((StringWrapper)(value)).wrapperValue;
      }
      else {
        value = StringWrapper.wrapString(self.signature);
      }
    }
    else if (slotname == Stella.SYM_STELLA_DOCUMENTATION) {
      if (setvalueP) {
        self.documentation = ((StringWrapper)(value)).wrapperValue;
      }
      else {
        value = StringWrapper.wrapString(self.documentation);
      }
    }
    else {
      { OutputStringStream stream000 = OutputStringStream.newOutputStringStream();

        stream000.nativeStream.print("`" + slotname + "' is not a valid case option");
        throw ((StellaException)(StellaException.newStellaException(stream000.theStringReader()).fillInStackTrace()));
      }
    }
    return (value);
  }

  public Surrogate primaryType() {
    { HookList self = this;

      return (Stella.SGT_STELLA_HOOK_LIST);
    }
  }

}

