// -*- Mode: Java -*-

/*---------------------------- BEGIN LICENSE BLOCK ---------------------------+
 |                                                                            |
 | Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
 |                                                                            |
 | The contents of this file are subject to the Mozilla Public License        |
 | Version 1.1 (the "License"); you may not use this file except in           |
 | compliance with the License. You may obtain a copy of the License at       |
 | http://www.mozilla.org/MPL/                                                |
 |                                                                            |
 | Software distributed under the License is distributed on an "AS IS" basis, |
 | WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
 | for the specific language governing rights and limitations under the       |
 | License.                                                                   |
 |                                                                            |
 | The Original Code is the PowerLoom KR&R System.                            |
 |                                                                            |
 | The Initial Developer of the Original Code is                              |
 | UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
 | 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
 |                                                                            |
 | Portions created by the Initial Developer are Copyright (C) 2002-2010      |
 | the Initial Developer. All Rights Reserved.                                |
 |                                                                            |
 | Contributor(s):                                                            |
 |                                                                            |
 | Alternatively, the contents of this file may be used under the terms of    |
 | either the GNU General Public License Version 2 or later (the "GPL"), or   |
 | the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
 | in which case the provisions of the GPL or the LGPL are applicable instead |
 | of those above. If you wish to allow use of your version of this file only |
 | under the terms of either the GPL or the LGPL, and not to allow others to  |
 | use your version of this file under the terms of the MPL, indicate your    |
 | decision by deleting the provisions above and replace them with the notice |
 | and other provisions required by the GPL or the LGPL. If you do not delete |
 | the provisions above, a recipient may use your version of this file under  |
 | the terms of any one of the MPL, the GPL or the LGPL.                      |
 |                                                                            |
 +----------------------------- END LICENSE BLOCK ---------------------------*/


// Version: Utils.java,v 1.4 2010/02/17 22:14:19 hans Exp

package edu.isi.powerloom.gui.common;

import java.io.*;

/**
 * Utility class for various things that don't fit anywhere else.
 *
 * @author Hans Chalupsky
 */
public class Utils {
    
    /**
     * Controls the quantity of debug and trace messages that are sent to
     * the console.  0 means "none", 1 is "low", 2 is "medium" and everything
     * else is "high".
     */
    public static int debugLevel = 0;

    public static java.io.PrintStream debugIO = System.out;

    public static void debugPrintln (int level, String message) {
        if (debugLevel > 0 && level <= debugLevel)
            debugIO.println(message);
    }

    public static void debugPrintStackTrace (int level, Throwable exception) {
        if (debugLevel > 0 && level <= debugLevel)
            exception.printStackTrace(debugIO);
    }

    public static String fileToString (String filename) throws AppException {
        try {
            File file = new File(filename);
            long fileLength = file.length();
            if (fileLength > Integer.MAX_VALUE)
                throw new Exception("File `" + filename +
                                    "' is too large for conversion into a string");
            char[] fileContent = new char[(int)fileLength];
            new FileReader(file).read(fileContent, 0, (int)fileLength);
            return new String(fileContent);
        }
        catch (Exception e) {
            throw new AppException(e);
        }
    }

    public static void stringToFile (String content, String filename)
        // Write `content' onto file `filename'.
        throws AppException {
        try {
            FileWriter writer = new FileWriter(filename);
            writer.write(content, 0, content.length());
            writer.close();
        }
        catch (Exception e) {
            throw new AppException(e);
        }
    }

    public static void exitGui (int code) {
        // Exit the GUI with status `code', unless we are running
        // embedded in PowerLoom, in which case we simply return.
        try {
            Class PLI = Class.forName("edu.isi.powerloom.PLI");
            if (PLI.getField("$POWERLOOM_INITIALIZEDp$").getBoolean(PLI)) {
                // we are running embedded, simply return to PowerLoom:
                debugPrintln(1, "Exiting embedded PowerLoom GUI...");
                return;
            }
        }
        catch(Exception e) {}
        // PLI class wasn't visible or initialized, exit all threads:
        debugPrintln(1, "Exiting PowerLoom GUI...");
        System.exit(code);
    }
}
