// -*- Mode: Java -*-

/*---------------------------- BEGIN LICENSE BLOCK ---------------------------+
 |                                                                            |
 | Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
 |                                                                            |
 | The contents of this file are subject to the Mozilla Public License        |
 | Version 1.1 (the "License"); you may not use this file except in           |
 | compliance with the License. You may obtain a copy of the License at       |
 | http://www.mozilla.org/MPL/                                                |
 |                                                                            |
 | Software distributed under the License is distributed on an "AS IS" basis, |
 | WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
 | for the specific language governing rights and limitations under the       |
 | License.                                                                   |
 |                                                                            |
 | The Original Code is the PowerLoom KR&R System.                            |
 |                                                                            |
 | The Initial Developer of the Original Code is                              |
 | UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
 | 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
 |                                                                            |
 | Portions created by the Initial Developer are Copyright (C) 2002-2010      |
 | the Initial Developer. All Rights Reserved.                                |
 |                                                                            |
 | Contributor(s):                                                            |
 |                                                                            |
 | Alternatively, the contents of this file may be used under the terms of    |
 | either the GNU General Public License Version 2 or later (the "GPL"), or   |
 | the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
 | in which case the provisions of the GPL or the LGPL are applicable instead |
 | of those above. If you wish to allow use of your version of this file only |
 | under the terms of either the GPL or the LGPL, and not to allow others to  |
 | use your version of this file under the terms of the MPL, indicate your    |
 | decision by deleting the provisions above and replace them with the notice |
 | and other provisions required by the GPL or the LGPL. If you do not delete |
 | the provisions above, a recipient may use your version of this file under  |
 | the terms of any one of the MPL, the GPL or the LGPL.                      |
 |                                                                            |
 +----------------------------- END LICENSE BLOCK ---------------------------*/


// Version: NewKBSaverModuleFrame.java,v 1.10 2010/04/19 20:10:01 hans Exp

package edu.isi.powerloom.gui.components;

import javax.swing.*;
import edu.isi.powerloom.gui.xmlobject.*;
import edu.isi.powerloom.gui.serverinterface.*;

/**
 * Frame for saving a module.  This frame presents a list of modules
 * and the presents a file saver dialog to select the path of the saved file.
 *
 * @author <a href=mailto:eric@metrotech-consulting.com>Eric Melz</a>
 */
public class NewKBSaverModuleFrame extends PLFrame {
    boolean isLocal;
    
    /** Creates new form NewKBSaverModuleFrame */
    public NewKBSaverModuleFrame(boolean local) {
	    this.isLocal = local;
        initComponents();
        setupModuleList();
    }
        
    void setupModuleList() {
	try {
            PLSurrogateContainer modules = KnowledgeManager.getInstance().getModules();
            moduleList.setModel(new PLListModel(modules.listifyTreeContainer()));
	} catch (Exception e) {
	    PowerloomApp.getInstance().handleException(e);
	}        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jScrollPane1 = new javax.swing.JScrollPane();
        moduleList = new javax.swing.JList();
        jPanel1 = new javax.swing.JPanel();
        NextButton = new javax.swing.JButton();
        CancelButton = new javax.swing.JButton();

        setTitle("Select Module to Save");
        jScrollPane1.setViewportView(moduleList);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        NextButton.setText("Next >");
        NextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                NextButtonActionPerformed(evt);
            }
        });

        jPanel1.add(NextButton);

        CancelButton.setText("Cancel");
        CancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                CancelButtonActionPerformed(evt);
            }
        });

        jPanel1.add(CancelButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        pack();
    }//GEN-END:initComponents

    private void CancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_CancelButtonActionPerformed
        setVisible(false);
    }//GEN-LAST:event_CancelButtonActionPerformed

    private void NextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_NextButtonActionPerformed
        PLModule selectedModule = (PLModule)moduleList.getSelectedValue();
        if (selectedModule == null) {
	    JOptionPane.showMessageDialog(PowerloomApp.getInstance(), "You must select a module before you proceed.", "Select Module", JOptionPane.WARNING_MESSAGE);
	    return;
        } else {
	    if (isLocal) {
		PLLocalFileBrowser fileBrowser = new PLLocalFileBrowser(PowerloomApp.getInstance(), selectedModule);
		fileBrowser.showSaveKBDialog();
	    } else {
                // disabled for now/ever (since we moved all the remote file support to attic
                // since it was a major security hole and used Sun proprietary source code).
		//PLRemoteFileBrowser fileBrowser = new PLRemoteFileBrowser(PowerloomApp.getInstance(), selectedModule);
		//fileBrowser.showSaveKBDialog();
	    }
        }
        setVisible(false);
    }//GEN-LAST:event_NextButtonActionPerformed
    
    
    /**
     * Implementation of PLFrame
     */
    public PLModule getModule() {
	return null;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton CancelButton;
    private javax.swing.JList moduleList;
    private javax.swing.JButton NextButton;
    // End of variables declaration//GEN-END:variables
    
}
